package com.hiramine.modelvieweretude;

import android.content.Context;
import android.opengl.GLSurfaceView;
import android.util.FloatMath;
import android.view.MotionEvent;

public class ModelViewerView extends GLSurfaceView
{
	private OpenGLModelRenderer	m_renderer;

	public ModelViewerView( Context context )
	{
		super( context );

		// Rendererの作成
		m_renderer = new OpenGLModelRenderer();

		// GLSurfaceViewにRendererをセット
		setRenderer( m_renderer );

		// 絶え間ないレンダリングではなく都度のレンダリング（setRenderer()よりも後に呼び出す必要あり）
		setRenderMode( GLSurfaceView.RENDERMODE_WHEN_DIRTY );

		// モデルの作成および登録
		m_renderer.setModel( makeModel() );
	}

	@Override
	public boolean onTouchEvent( MotionEvent event )
	{
		int action = event.getAction();
		int pointcount = event.getPointerCount();
		final float fX = event.getX();
		final float fY = event.getY();

		switch( action & MotionEvent.ACTION_MASK )
		{
		// トラッキングの開始
		case MotionEvent.ACTION_DOWN:
		case MotionEvent.ACTION_POINTER_DOWN:
			if( 1 == pointcount )
			{ // １つの指でドラッグ＝＞スピン
				m_renderer.beginTracking( fX, fY, ETRACKINGMODE.TM_SPIN );
			}
			else if( 2 == pointcount )
			{ // ２つの指でドラッグ＝＞ズームもしくはパン
				// ２つの指が遠いときは、ズーム、近い時は、パン
				float fX1 = event.getX( 1 );
				float fY1 = event.getY( 1 );
				final float fDistance = FloatMath.sqrt( ( fX1 - fX ) * ( fX1 - fX ) + ( fY1 - fY ) * ( fY1 - fY ) );
				if( 150 < fDistance )
				{ // ズーム
					m_renderer.beginTracking( -fDistance, -fDistance, ETRACKINGMODE.TM_ZOOM );
				}
				else
				{ // パン
					m_renderer.beginTracking( fX, fY, ETRACKINGMODE.TM_PAN );
				}
			}
			break;
		// トラッキングの終了
		case MotionEvent.ACTION_UP:
		case MotionEvent.ACTION_POINTER_UP:
			m_renderer.endTracking();
			break;
		// トラッキング
		case MotionEvent.ACTION_MOVE:
			if( 2 == pointcount
					&& ETRACKINGMODE.TM_ZOOM == m_renderer.getTrackingMode() )
			{ // ズームは、１つ目の指の座標値ではなく、２つの指の距離で処理する。
				float fX1 = event.getX( 1 );
				float fY1 = event.getY( 1 );
				final float fDistance = FloatMath.sqrt( ( fX1 - fX ) * ( fX1 - fX ) + ( fY1 - fY ) * ( fY1 - fY ) );
				m_renderer.doTracking( -fDistance, -fDistance );
				requestRender(); // 再描画
			}
			else
			{ // 通常は、１つ目の指の座標値で処理する。
				m_renderer.doTracking( fX, fY );
				requestRender(); // 再描画
			}
			break;
		}

		return true;
	}

	// モデルの作成
	private Model makeModel()
	{
		float[] afVertex = {
				-5.0f, -5.0f, -5.0f,
				5.0f, -5.0f, -5.0f,
				-5.0f, 5.0f, -5.0f,

				5.0f, -5.0f, -5.0f,
				5.0f, 5.0f, -5.0f,
				-5.0f, 5.0f, -5.0f,

				5.0f, -5.0f, -5.0f,
				5.0f, -5.0f, 5.0f,
				5.0f, 5.0f, -5.0f,

				5.0f, -5.0f, 5.0f,
				5.0f, 5.0f, 5.0f,
				5.0f, 5.0f, -5.0f,

				5.0f, -5.0f, 5.0f,
				-5.0f, -5.0f, 5.0f,
				5.0f, 5.0f, 5.0f,

				-5.0f, -5.0f, 5.0f,
				-5.0f, 5.0f, 5.0f,
				5.0f, 5.0f, 5.0f,

				-5.0f, -5.0f, 5.0f,
				-5.0f, -5.0f, -5.0f,
				-5.0f, 5.0f, 5.0f,

				-5.0f, -5.0f, -5.0f,
				-5.0f, 5.0f, -5.0f,
				-5.0f, 5.0f, 5.0f,

				-5.0f, 5.0f, -5.0f,
				5.0f, 5.0f, -5.0f,
				-5.0f, 5.0f, 5.0f,

				5.0f, 5.0f, -5.0f,
				5.0f, 5.0f, 5.0f,
				-5.0f, 5.0f, 5.0f,

				-5.0f, -5.0f, 5.0f,
				5.0f, -5.0f, 5.0f,
				-5.0f, -5.0f, -5.0f,

				5.0f, -5.0f, 5.0f,
				5.0f, -5.0f, -5.0f,
				-5.0f, -5.0f, -5.0f };

		return new Model( afVertex );
	}
}
